%set based graphs
setBasedGraphs=true;
global debugging
debugging=false;

function E=createSetEdge(i1,i2,map1,map2)
%i1 and i2 are the indices of the set vertices connected by E
%map1 is a map that relates each edge with the first set vertex
%map2 is a map that relates each edge with the second set vertex
  E=struct("i1",i1,"i2",i2,"map1",map1,"map2",map2);
end

function sbg=createSBGraph(V,E)
  %V is a list of set vertices, represented by sets
  %E is a list of set edges.
  sbg=struct("setVertex",V,"setEdge",E);
end


function v=verticesSBGraph(S)
%returns the set of scalar vertices of a SB Graph
  v=S.setVertex(1);
  for i=2:length(S.setVertex)
    v=setUnion(v,S.setVertex(i));
  end
end

function [s,h1,spos]=succV(S,i)
 %s is the list of successors of i--th Set-Vertex of S
 %h1 is the list of set vertices
 %spos contains the position of successor (1 for i1, 2 for i2)
 %This is for non directed Graph
 s=[];
 h1=[];
 spos=[];
 for h=1:length(S.setEdge)
   if S.setEdge(h).i1==i
     s=[s S.setEdge(h).i2];
     h1=[h1 h];
     spos=[spos 2]; 
   elseif S.setEdge(h).i2==i
     s=[s S.setEdge(h).i1];
     h1=[h1 h];
     spos=[spos 1]; 
   end
 end
end


function Rmap=connectComp(SBG)
   %connected components of SB Graph S. 
   %returns an array of maps from set vertices to connected components
   %A connected component is the set of all components with the same image on Rmap
   S=SBG;
   domR=verticesSBGraph(SBG); 
   Id=idMap(domR);
   Rmap=Id; %we initially assume that all vertices are disconnected
   %We'll have all maps in one
   Emap1=SBG.setEdge(1).map1;
   Emap2=SBG.setEdge(1).map2;
   for h=2:length(SBG.setEdge)
     Emap1=combinePWLMaps(SBG.setEdge(h).map1,Emap1);
     Emap2=combinePWLMaps(SBG.setEdge(h).map2,Emap2);
   end
       debugDisp("Emap1:")
       debugMap(Emap1)
       debugDisp("Emap2:")
       debugMap(Emap2)
   
   lastImage=[]; %last image of Rmap
   newImage=domR; %new image of Rmap
   diffImage=domR; %current difference between images
   while !isempty(diffImage) %e should iterate until the image of Rmap become constant
       ERmap1=compMap(Rmap,Emap1);
       debugDisp("ERmap1:")
       debugMap(ERmap1)
       ERmap2=compMap(Rmap,Emap2);
       debugDisp("ERmap2:")
       debugMap(ERmap2)
       Rmap1=minAdjMap(ERmap1,ERmap2);
       debugDisp("Rmap1:")
       debugMap(Rmap1)
       Rmap2=minAdjMap(ERmap2,ERmap1);
       debugDisp("Rmap2:")
       debugMap(Rmap2)
       Rmap1=combinePWLMaps(Rmap1,Rmap); %extend domain
       Rmap2=combinePWLMaps(Rmap2,Rmap); %extend domain
       Rmapnew=minMap(Rmap1,Rmap2);
       lastImage=newImage;
       newImage=imagePWLinearMap(domR,Rmapnew);
       diffImage=setMinus(lastImage,newImage);
       if !isempty(diffImage)
         Rmap=Rmapnew;
         debugDisp("Rmap:")       
         debugMap(Rmap)
         Rmap=mapInf(Rmap);
         debugDisp("Rmapinf")
         debugMap(Rmap)
         newImage=imagePWLinearMap(domR,Rmap);
       end  
   end  
end


function code=genFlatterCodeMD(Rmap)
  %code generation for flatter multidimensional
  code="";
  asc=104; %char(104+1)=i
  VCmapN=atomizeMap(Rmap);
  VCDom=domPWLMap(VCmapN);
  VCIm=imagePWLinearMap(VCDom,VCmapN); %image split into atomic sets with identical treatment
  for i=1:length(VCIm.interval)
    VCImi=atomicSet(VCIm,i); %i-th atomic set of the image (CC)
    VCDomi=preImagePWLinearMap(VCImi,VCmapN); %domain of the CC (atomic sets)
    %code for efforts
    VCDomiaux=setMinus(VCDomi,VCImi); %exclude image set (CC)
    if !isempty(VCDomiaux)
      for j=1:length(VCDomiaux.interval) %traverse atomic sets
         itdom=[];   
         itim=[];
         for d=1:length(VCDomiaux.interval(j).start) %traverse dimensions
            it=char(asc+d); %i,j,k, etc
            if VCImi.interval(1).start(d)==VCImi.interval(1).end(d) %single component in the image
              iti=mat2str(VCImi.interval(1).start(d)); %iterator on image 
            else
              iti=strcat(it,"-",mat2str(VCDomiaux.interval(j).start(d)-VCImi.interval(1).start(d)));  
            end 
            if d>1
              itdom=strcat(itdom,",");
              itim=strcat(itim,",");
            end
            itdom=strcat(itdom,it); % create i,j,k expression for domain
            itim=strcat(itim,iti); % create i,j,k expression for domain
         end
         code=cstrcat(code,"for ",itdom," in ",set2str(atomicSet(VCDomiaux,j)),char(10));
         code=cstrcat(code,"  effort(",itdom,") = effort(",itim,")",char(10));
         code=cstrcat(code,"end",char(10));
      end
    end
    %code for flows
    itim=[];
    itdom={};
    itfor={};
    for d=1:length(VCImi.interval.start) %traverse dimensions of CC
      it=char(asc+d); %i,j,k, etc
      if d>1 %add a comma
         itim=strcat(itim,",");
      end
      itim=strcat(itim,it); % create i,j,k expression for image
      if !isempty(VCDomiaux) % create expressions for domain
        for j=1:length(j=1:length(VCDomiaux.interval))
           if d>1 %add a comma
             itdom{j}=strcat(itdom{j},",");
           else
             itdom{j}="";
             itfor{j}="";
           end
           if VCImi.interval(1).start(d)==VCImi.interval(1).end(d)&&VCDomiaux.interval(j).start(d)<VCDomiaux.interval(j).end(d) %single component in the image
               itdom{j}=strcat(itdom{j},it,"1");
               itfor{j}=cstrcat(itfor{j},", for ",it,"1 in ",interval2str(uniInterval(VCDomiaux.interval(j),d)));
           else
               itdom{j}=strcat(itdom{j},it,"+",mat2str(VCDomiaux.interval(j).start(d)-VCImi.interval(1).start(d)));
           end
        end
      end
    end
      code=cstrcat(code,"for ",itim," in ",set2str(VCImi),char(10));
      code=cstrcat(code,"  flow(",itim,")");
      if !isempty(VCDomiaux) % create expressions for domain
        for j=1:length(j=1:length(VCDomiaux.interval))
          if isempty(strfind(itfor{j},"for"))
            code=strcat(code," + flow(",itdom{j},")");
            
          else
            code=strcat(code," + sum(flow(",itdom{j},")",itfor{j},")");
          end  
        end  
      end
      code=cstrcat(code," = 0",char(10));
      code=cstrcat(code,"end",char(10));

    end
end

function debugMap(d)
  global debugging
  if debugging
    showMap(d)
  end  
end

function debugDisp(d)
  global debugging
  if debugging
    display(d)
  end
end

