intensionalSets;
SBGraphs;

function sbg=RCGraph2(N)
  %create SBG for RC circuit with N sections
  %Connectots are
  % S.p,S.n,G.p,R(i).p,R(i).n,C(i).p,C(i).n
  %create set vertices
  %offset in global set
  offSp=0;
  offSn=1;
  offGp=2;
  offRp=N;
  offRn=2*N;
  offCp=3*N;
  offCn=4*N;
  Sp=createSet(createInterval(1+offSp,1,1+offSp));
  Sn=createSet(createInterval(1+offSn,1,1+offSn));
  Gp=createSet(createInterval(1+offGp,1,1+offGp));
  Rp=createSet(createInterval(1+offRp,1,N+offRp));
  Rn=createSet(createInterval(1+offRn,1,N+offRn));
  Cp=createSet(createInterval(1+offCp,1,N+offCp));
  Cn=createSet(createInterval(1+offCn,1,N+offCn));
  % Connections are
  % (S.p,R(1).p), (S.n,G.p), (R(i).n,C(i).p), (R(i).n,R(i+1).p), (C(1).n,G.p), (C(i).n,C(i-1).n) 
  % create set edges
  offE1=0;
  offE2=1;
  offE3=2;
  offE4=2+N;
  offE5=2+2*N-1;
  offE6=2+2*N;
  

  domE1=createSet(createInterval(1+offE1,1,offE2));
  mapE1Sp=createPWLinearMap(domE1,createLinearMap(0,1+offSp));
  mapE1Rp=createPWLinearMap(domE1,createLinearMap(0,1+offRp));
  E1=createSetEdge(1,4,mapE1Sp,mapE1Rp);

  domE2=createSet(createInterval(1+offE2,1,offE3));
  mapE2Sn=createPWLinearMap(domE2,createLinearMap(0,1+offSn));
  mapE2Gp=createPWLinearMap(domE2,createLinearMap(0,1+offGp));
  E2=createSetEdge(2,3,mapE2Sn,mapE2Gp);

  domE3=createSet(createInterval(1+offE3,1,offE4));
  mapE3Rn=createPWLinearMap(domE3,createLinearMap(1,offRn-offE3));
  mapE3Cp=createPWLinearMap(domE3,createLinearMap(1,offCp-offE3));
  E3=createSetEdge(5,6,mapE3Rn,mapE3Cp);

  domE4=createSet(createInterval(1+offE4,1,offE5));
  mapE4Rn=createPWLinearMap(domE4,createLinearMap(1,offRn-offE4));
  mapE4Rp=createPWLinearMap(domE4,createLinearMap(1,1+offRp-offE4));
  E4=createSetEdge(5,4,mapE4Rn,mapE4Rp);

  domE5=createSet(createInterval(1+offE5,1,offE6));
  mapE5Cn=createPWLinearMap(domE5,createLinearMap(1,offCn-offE5));
  mapE5Gp=createPWLinearMap(domE5,createLinearMap(0,1+offGp));
  E5=createSetEdge(7,3,mapE5Cn,mapE5Gp);
  
  domE6=createSet(createInterval(1+offE6,1,1+offE6+N-2));
  mapE6Cn=createPWLinearMap(domE6,createLinearMap(1,offCn-offE6));
  mapE6Cn2=createPWLinearMap(domE6,createLinearMap(1,offCn-offE6+1));
  E6=createSetEdge(7,7,mapE6Cn,mapE6Cn2);
  
  sbg=createSBGraph([Sp,Sn,Gp,Rp,Rn,Cp,Cn],[E1,E2,E3,E4,E5,E6]);
end

S=RCGraph2(1000);
tic
Rmap=connectComp(S);
toc
display("Connected components map:")
showMap(Rmap)
code=genFlatterCodeMD(Rmap);
display("Output Code:")
display(code)


