intensionalSets;
SBGraphs;

function sbg=TwoDimGraph(N,M)
  %create SBG for s network with NxM sections
  %Connectots are
  % S.p, S.n, G.p
  % C.l,C.r,C.u,C.d (left, right, up, down)
  %create set vertices

  %offset
  offSp1=0;
  offSn1=1;
  offGp1=2;
  offCl1=N;
  offCr1=2*N;
  offCu1=3*N;
  offCd1=4*N;

  offSp2=0;
  offSn2=1;
  offGp2=2;
  offCl2=M;
  offCr2=2*M;
  offCu2=3*M;
  offCd2=4*M;
  
  Sp=createSet(createInterval([1+offSp1 1+offSp2],[1 1],[1+offSp1 1+offSp2]));
  Sn=createSet(createInterval([1+offSn1 1+offSn2],[1 1],[1+offSn1 1+offSn2]));
  Gp=createSet(createInterval([1+offGp1 1+offGp2],[1 1],[1+offGp1 1+offGp2]));

  Cl=createSet(createInterval([1+offCl1 1+offCl2],[1 1],[offCr1 offCr2]));
  Cr=createSet(createInterval([1+offCr1 1+offCr2],[1 1],[offCu1 offCu2]));
  Cu=createSet(createInterval([1+offCu1 1+offCu2],[1 1],[offCd1 offCd2]));
  Cd=createSet(createInterval([1+offCd1 1+offCd2],[1 1],[N+offCd1 M+offCd2]));
  
  % Connections are
  % (C.r(i,j),c.l(i,j+1)) -> (N x M-1 connections), 
  % (C.d(i,j),C.u(i+1,j)) -> (N-1 x M connections) 
  % (C.l(i,1),C.r(i,M))  ->  (N x 1 connections)
  % (S.p,C.u(1,j)), ->      (1 x M connections) 
  %(G.p,C.d(N,j))  -> (1 x M connections)
  % (S.n,G.p)  -> (1 x 1 connections)
  offE11=0; 
  offE12=0;
  offE21=offE11+N;
  offE22=offE12+M-1;
  offE31=offE21+N-1;
  offE32=offE22+M;
  offE41=offE31+N;
  offE42=offE32+1;
  offE51=offE41+1;
  offE52=offE42+M;
  offE61=offE51+1;
  offE62=offE52+M;
  offE71=offE61+1;
  offE72=offE62+1;
  
  domE1=createSet(createInterval([1+offE11 1+offE12],[1 1],[offE21 offE22]));
  mapE1Cl=createPWLinearMap(domE1,createLinearMap(diag([1,1]),[offCl1-offE11;offCl2-offE12+1]));
  mapE1Cr=createPWLinearMap(domE1,createLinearMap(diag([1,1]),[offCr1-offE11;offCr2-offE12]));
  E1=createSetEdge(4,5,mapE1Cl,mapE1Cr);
 
  domE2=createSet(createInterval([1+offE21 1+offE22],[1 1],[offE31 offE32]));
  mapE2Cu=createPWLinearMap(domE2,createLinearMap(diag([1,1]),[offCu1-offE21+1;offCu2-offE22]));
  mapE2Cd=createPWLinearMap(domE2,createLinearMap(diag([1,1]),[offCd1-offE21;offCd2-offE22]));
  E2=createSetEdge(6,7,mapE2Cu,mapE2Cd);

  domE3=createSet(createInterval([1+offE31 1+offE32],[1 1],[offE41 offE42]));
  mapE3Cl=createPWLinearMap(domE3,createLinearMap(diag([1,1]),[offCl1-offE31;offCl2-offE32]));
  mapE3Cr=createPWLinearMap(domE3,createLinearMap(diag([1,1]),[offCr1-offE31;offCr2-offE32+M-1]));
  E3=createSetEdge(4,5,mapE3Cl,mapE3Cr);

  domE4=createSet(createInterval([1+offE41 1+offE42],[1 1],[offE51 offE52]));
  mapE4Cu=createPWLinearMap(domE4,createLinearMap(diag([1,1]),[offCu1-offE41;offCu2-offE42]));
  mapE4Sp=createPWLinearMap(domE4,createLinearMap(diag([0,0]),[offSp1+1;offSp2+1]));
  E4=createSetEdge(6,1,mapE4Cu,mapE4Sp);

  domE5=createSet(createInterval([1+offE51 1+offE52],[1 1],[offE61 offE62]));
  mapE5Cd=createPWLinearMap(domE5,createLinearMap(diag([1,1]),[offCd1-offE51+N-1;offCd2-offE52]));
  mapE5Gp=createPWLinearMap(domE5,createLinearMap(diag([0,0]),[offGp1+1;offGp2+1]));
  E5=createSetEdge(7,3,mapE5Cd,mapE5Gp);
 
  domE6=createSet(createInterval([1+offE61 1+offE62],[1 1],[offE71 offE72]));
  mapE6Sn=createPWLinearMap(domE6,createLinearMap(diag([0,0]),[offSn1+1;offSn2+1]));
  mapE6Gp=createPWLinearMap(domE6,createLinearMap(diag([0,0]),[offGp1+1;offGp2+1]));
  E6=createSetEdge(2,3,mapE6Sn,mapE6Gp);

  sbg=createSBGraph([Sp,Sn,Gp,Cl,Cr,Cu,Cd],[E1,E2,E3,E4,E5,E6]);
  

end

S=TwoDimGraph(1000,100);
tic
Rmap=connectComp(S);
toc
display("Connected components map:")
showMap(Rmap)
code=genFlatterCodeMD(Rmap);
display("Output Code:")
display(code)
