%functions for manipulation of intervals and sets.
aux=true;

function int1 = createNoCropInterval(Start,Step,End);
  %Create a new Interval of the form [Start:Step:End]
  int1=struct("start",Start,"step",Step,"end",End);
end;

function int1 = createInterval(Start,Step,End);
  %Create a new Interval of the form [Start:Step:End] and adjusts End
  int1=struct("start",Start,"step",Step,"end",End);
  int1=cropInterval(int1);
end;

function int2=cropUniInterval(int1)
%adjusts int1.end to coincide with step for unidimensional interval
  int2=int1;
  if int2.end<Inf
    r=rem(int2.end-int2.start,int2.step);
    int2.end=int2.end-r;
  end  
end

function int2=cropInterval(int1)
  int2=int1;
  for i=1:length(int1.start)
    intaux=cropUniInterval(uniInterval(int1,i));
    int2.end(i)=intaux.end;
  end
end

function int1 = createUnitInterval(N)
  %creates a unit interval of dimension N
  int1=createInterval(ones(1,N),ones(1,N),ones(1,N));
end

function int1 = createFullInterval(N)
   int1=createInterval(ones(1,N),ones(1,N),Inf*ones(1,N));  
end

function int2=uniInterval(int1,i)
%returns unidimensional interval in the  i-th dimension
  int2=createNoCropInterval(int1.start(i),int1.step(i),int1.end(i));
end

function str=uniInterval2str(int1)
  %convert unidimensional interval to string
  if isempty(int1)
    str = "emptyInterval";
  else
      str=strcat("[",mat2str(int1.start),":",mat2str(int1.step),":",mat2str(int1.end),"]");
  end
end


function str=interval2str(int1)
  %convert interval to string
  if isempty(int1) 
    str = "emptyInterval";
  else  
    str=uniInterval2str(uniInterval(int1,1));
    for i=2:length(int1.start)
      str=strcat(str,"x",uniInterval2str(uniInterval(int1,i)));
    end
  end  
end

function showInterval(int1)
  %display interval
  disp(interval2str(int1));
end


function int3=intCrossProduct(int1,int2);  
%cross product of intervals 
  int3.start=[int1.start,int2.start];
  int3.step=[int1.step,int2.step];
  int3.end=[int1.end,int2.end];
end

function int3=replaceDimension(int1,int2,i);  
%replaces the i-th dimension of int1 with int2
  if !isempty(int2)&&!isempty(int1)
    int3 =int1;
    int3.start(i)=int2.start;
    int3.step(i)=int2.step;
    int3.end(i)=int2.end;
  else
    int3=[];
  end  
end




function inter=interUniInterval(int1,int2)
  %computes the intersection of two unidimensional intervals
  if (int1.end<int2.start)||(int2.end<int1.start) 
    inter=[]; %empty interval
  else
    newStep=lcm(int1.step,int2.step); %least common multiplier
    startInt1=[int1.start:int1.step:min(int1.start+newStep-1,int1.end)]; %first newStep/step components    
    startInt2=[int2.start:int2.step:min(int2.start+newStep-1,int2.end)]; %first newStep/step components
    remainders1=rem(startInt1,newStep); %remainder of integer division of first components by newStep
    remainders2=rem(startInt2,newStep); %remainder of integer division of first components by newStep
    [c,i1,i2]=intersect(remainders1,remainders2); %intersection of remainders (and their indices)
    if isempty(c) 
      inter=[]; %if the remainders do not intersect, the intersection is empty
    else
      newStart=max(startInt1(i1),startInt2(i2));  %first component with common remainder
      newEnd=min(int1.end,int2.end); %largest possible last component of the intersection
      inter=createInterval(newStart,newStep,newEnd); %intersection interval
      newEnd=inter.end;
      if newStart>newEnd %check that the intersection interval is not empty
        inter=[];
      end 
    end   
  end   
end

function inter=interInterval(int1,int2)
 %Intersection of multi dimensional intervals
  if (isempty(int1)|| isempty(int2))
    interdim=[];
  else  
    interdim=interUniInterval(uniInterval(int1,1),uniInterval(int2,1)); %intersect first dimension
  end
  if isempty(interdim)
    inter=[];
  else  
    inter=interdim; %first dimension of the intersection
    for i=2:length(int1.start) 
      interdim=interUniInterval(uniInterval(int1,i),uniInterval(int2,i)); %intersect i-th dimension
      if isempty(interdim)
        inter=[];                  %if a dimension has empty intersection, the overall intersection is empty
        break;
      else
       inter=intCrossProduct(inter,interdim);  %join with ith dimension of the intersection
      end  
    end 
  end  
end

function ints=removeUniInterval(int1,int2)
%returns an array of intervals corresponding to the removal of int2 from int1 (unidimensional intervals)
  int1a=cropUniInterval(int1); %adjust end of interval in case it does not coincide with the step
  int2a=interUniInterval(int1a,int2); %we substract the intersection of both intervals from int1
  if isempty(int2a)
    ints=int1a; %nothing to remove  
  else 
    nints=0; %resulting number of intervals
    if int2a.start>int1a.start %left most interval
      intaux=createInterval(int1a.start,1,int2a.start-1);
      ints=interUniInterval(int1a,intaux); %interval of int1a before int2a.start
      nints=1; 
    end  
    if int2a.end-int2a.start>int2a.step 
      numints=int2a.step/int1a.step-1; %number of intervals of int1a not in the intersection with int2a
    else
       numints=0;
    end   
    for i=1:numints
      newint=createInterval(int2a.start+i*int1a.step,int2a.step,int2a.end); %int2a.start+i*int1a.step is not removed
      ints(nints+1)=newint; %add not removed interval
      nints=nints+1;
    end
    if int2a.end<int1a.end %right most interval
      intaux=createInterval(int2a.end+1,1,int1a.end);
      ints(nints+1)=interUniInterval(int1a,intaux);
      nints=length(ints); %interval of int1a after int2a.end
    end
    if nints==0 %no interval was added in the process
      ints=[]; %empty interval
    end  
  end  
end

function outints=removeInterval(int1,int2)
%returns an array of intervals corresponding to the removal of int2 from int1
  N=length(int1.start); %dimension
  int2a=interInterval(int1,int2); %we only remove the intersection
  if isempty(int2a)
    outints=int1;
  else 
    nmints=0;
    for i=1:N %dimension of intervals
      ints=removeUniInterval(uniInterval(int1,i),uniInterval(int2a,i)); %computes the intervals that remain in the ith dimension
      for j=1:length(ints) %number of intervals
        auxint=createUnitInterval(N);
        for k=1:N %create a multidimensional interval for each interval
          if k<i 
            auxint=replaceDimension(auxint,uniInterval(int2a,k),k); %if the dimension to add is previous to the removed dimension, add the removed interval
          elseif k==i
            auxint=replaceDimension(auxint,ints(j),k); %if the dimension to add is the removed dimension, add the remaining interval
          else
            auxint=replaceDimension(auxint,uniInterval(int1,k),k); %if the dimension to add is greater than the removed dimension, keep the original interval
          end  
        end  
        nmints=nmints+1;
        outints(nmints)=auxint; %add new interval to the output array
      end  
    end
    if nmints==0 
      outints=[];
    end  
  end  
end


function int2=uniIntervalAffine(int1,gain,offset)
 %Affine function of a unidimensional interval
  int2=int1;
  if gain>0 
    int2.start=int1.start*gain+offset;
    int2.end=int1.end*gain+offset;
    int2.step=int1.step*gain;
    if int2.step<1 %non integer gain
      int2.step=1; 
      int2.start=ceil(int2.start);
      int2.end=floor(int2.end);
    end
      
    if int2.start<0
      int2.start=int2.start+int2.step*(1+floor(abs(int2.start)/int2.step));
    end
    if int2.start==int2.end
      int2.step=1;
    elseif int2.end<int2.start
      int2=[];
    end  
  else
    if offset>0 
      int2.start=offset;
      int2.step=1;
      int2.end=offset;
    else
      int2=[];
    end  
  end
end

function isinint=isInInterval(c,int1)
  %returns true if c belongs to interval int1.
  isinint=true;
  for i=1:length(c)
    isinint=isinint&& c(i)>=int1.start(i) && c(i)<=int1.end(i) && rem(c(i),int1.step(i))==rem(int1.start(i),int1.step(i));
  end
end


%%%%%%%%%%%%
%Sets
%%%%%%%%%%%%%

function newSet = createSet(ints);
%creates a Set. A Set contains a list of Intervals
  if isempty(ints)
    newSet=[];
  else  
    newSet = struct("interval",ints);
  end;  
end;


function newSet = createUnitSet(N)
  %creates atomic unit set of dimension N 
  newSet=createSet(createUnitInterval(N));
end

function newSet = createFullSet(N)
  %creates a full atomic set of dimension N
  newSet=createSet(createFullInterval(N));
end

function newSet= singleton(v);
  N=length(v);
  int1=createInterval(ones(1,N).*v,ones(1,N),ones(1,N).*v);
  newSet=createSet(int1);
end

function newSet = atomicSet(set1,i)
  %atomic set with the i--th interval of set1.
  newSet=createSet(set1.interval(i));
end

function newSet = removeAtomicSet(set1,i)
  %remove i--th atomic set of set1.
  if length(set1.interval)==1
    newSet=[];
  else
    newSet=set1;
    if i<length(set1.interval)
      newSet.interval=[set1.interval(1:i-1),set1.interval(i+1:length(set1.interval))];
    else
      newSet.interval=[set1.interval(1:i-1)];
    end
  end  
end


function strset=atomicSet2str(set1)
%string representation of atomic set
    if isempty(set1)
       strset="emptySet"
    else
       strset=strcat("{",interval2str(set1.interval),"}");
    end   
end

function strset=set2str(myset)
%string representation of set
    if isempty(myset)
       strset="emptySet";
    else
       strset=atomicSet2str(atomicSet(myset,1)); %first interval
       for i=2:length(myset.interval)
         strset=strcat(strset,"U",atomicSet2str(atomicSet(myset,i)));
       end  
    end   
end



function showSet(myset)
  disp(set2str(myset))
end


function set3=interAtomicSet(set1,set2)
%intersection of atomic sets
  int3=interInterval(set1.interval,set2.interval);
  if isempty(int3)
    set3=[];
  else
    set3=createSet(int3);
  end  
end


function outset=appendSet(inset,aset)
%adds a set to an existing set
  if isempty(inset)
    outset=aset;
  elseif isempty(aset)
    outset=inset;  
  else
    outset=createSet([inset.interval,aset.interval]);
  end  
end

function outset=interSet(set1,set2)
%intersection of sets
  outset=[];
  if !(isempty(set1)||isempty(set2)) 
    for i=1:length(set1.interval)
      for j=1:length(set2.interval)
        outset=appendSet(outset,interAtomicSet(atomicSet(set1,i),atomicSet(set2,j)));
      end  
    end
  end  
end

function asets=atomicSetMinus(aset1,aset2)
 %remove atomic set ase2 from aset1. Returns a set
  ints=removeInterval(aset1.interval,aset2.interval);
  if isempty(ints)
    asets=[];
  else  
     asets=createSet(ints);
  end  
end

function outset=setMinus(set1,set2)
  outset=[];
  set2a=interSet(set1,set2);
  if !isempty(set2a)
    for i=1:length(set1.interval) %number of atomic Sets
      sets=atomicSet(set1,i); %we will remove all atomic sets of set2a from sets    
      for j=1:length(set2a.interval)
        newsets=[];           %result of sets-set2a.atomicSet(j)
        if !isempty(sets)
          for k=1:length(sets.interval)  %number of atomic sets of sets (originally 1, but then changes)
            newsets=appendSet(newsets,atomicSetMinus(atomicSet(sets,k),atomicSet(set2a,j))); %remove from each atomic Set of sets
          end
          sets=newsets; %sets is now the result of removing set2a.atomicSet(j) from sets.
        end  
      end
      outset=appendSet(outset,sets); %add sets to the output set
    end
  else
    outset=set1; %set1-emptyset is set1 
  end  
end

function outset=setUnion(set1,set2)
  set2a=setMinus(set2,set1); %first remove set1 from set2 to compute on disjoints sets
  outset=set1;
  if !isempty(set2a)
    outset=appendSet(outset,set2a);   
  end  
end

function outaset=atomicSetProduct(aset1,aset2)
%set product between two atomic sets. Returns a set
  minterval=intCrossProduct(aset1.interval,aset2.interval);
  outaset=createSet(minterval);
end

function outset=setProduct(set1,set2)
  outset=[];  
  for i=1:length(set1.interval);
    for j=1:length(set2.interval);
      outset=setUnion(outset,atomicSetProduct(atomicSet(set1,i),atomicSet(set2,j)));
    end
  end
end

function minc=minComponentSet(set1)
  %minimum component of a Set
  %we say that a<b if a(1)<b(1) or a(k)=b(k) k=1,dots,m and a(k+1)<b(k+1)
  minc=set1.interval(1).start; %we propose first the start of the first atomic set of set1
  for i=2:length(set1.interval)
    a1=set1.interval(i).start < minc; %a1 is 1 where minc is larger
    a2=set1.interval(i).start > minc; %a2 is 1 where minc is smaller
    [b1,p1]=max([a1 1]);%p1 is the first position where a1 is 1
    [b2,p2]=max([a2 1]);%p2 is the first position where a2 is 1
    if p1 < p2 %a1 is true begore a2
      minc=set1.interval(i).start;
    end  
  end
end

function isinset=isInSet(c,set1)
  isinset=false;
  for i=1:length(set1.interval)
    isinset=isInInterval(c,set1.interval(i));
    if isinset
      break
    end  
  end
end

function aseti=dimAtomicSet(aset,i)
%returns the i-th dimension if atomic set
  aseti=createSet(uniInterval(aset.interval,i));
end

function seti=dimSet(set1,i)
  %i-th dimension of a set
  seti=dimAtomicSet(atomicSet(set1,1),i);
  for j=2:length(set1.interval)
    asetj=dimAtomicSet(atomicSet(set1,j),i);
    seti.interval=[seti.interval,asetj.interval];
  end
end

%%%%%%%%%%%%%%% Maps

function newlmap=createLinearMap(A,B)
  %a linear map has a rational matrix A (diagonal NxN) and a rational vector B (1xN)
  %y=Ax+B
  newlmap=struct("gain",A,"offset",B);
end

function outlmap=compLinearMap(lmap2,lmap1)
  %Composite linear map: outlmap= lmap1(lmap2)
  outlmap=createLinearMap(lmap2.gain*lmap1.gain,lmap2.gain*lmap1.offset+lmap2.offset);
  newgain=outlmap.gain;
end

function lmap2=invLinearMap(lmap1)
  %inverse of a diagonal linear map
  lmap2=lmap1;
  for i=1:length(lmap1.offset)
    if lmap1.gain(i,i)!=0
        lmap2.gain(i,i)=1/lmap1.gain(i,i);
        lmap2.offset(i)=-lmap1.offset(i)/lmap1.gain(i,i);
     else
        lmap2.gain(i,i)=Inf;
        lmap2.offset(i)=-Inf;
     end   
  end
end;

function y=sameMap(lmap1,lmap2)
%true if both maps have identical gain and offset
  y1=sum(diag(lmap1.gain==lmap2.gain))==length(lmap1.offset);
  y2=sum(lmap1.offset==lmap2.offset)==length(lmap1.offset);
  y=y1&&y2;
end


function newmap=createPWLinearMap(dom,lmap)
%A Piecewise Linear Linear map has a list of disjoint sets and a list of Linear maps
%newmlmap(v)=lmap(k)(v) iff v belongs to dom(k) 
  newmap=struct("domain",dom,"linearMap",lmap);
end

function str=linearMap2str(lmap)
  str=strcat("y = ",mat2str(lmap.gain,5)," * x + ",mat2str(lmap.offset,5));
end

function str=map2str(pwlmap)
  if !isempty(pwlmap)
    str=strcat(linearMap2str(pwlmap.linearMap(1))," if x in",set2str(pwlmap.domain(1)));
    for i=2:length(pwlmap.domain)
      str=strcat(str,char(10),linearMap2str(pwlmap.linearMap(i))," if x in",set2str(pwlmap.domain(i)));
    end
  else
    str="emptyMap";
  end  
end

function showLinearMap(lmap)
  display(linearMap2str(lmap))
end

function showMap(pwlmap)
  display(map2str(pwlmap))
end


function idmap=idMap(set1)
 %identity map in set1
 N=length(set1.interval(1).start); %dimension
 lmap=createLinearMap(eye(N,N),zeros(N,1));
 idmap=createPWLinearMap(set1,lmap);
end

function imaset=imageAtomicLinearMap(aset,lmap)
  %image of atomic set aset throug linear map lmap
  %returns a set
  if !isempty(aset)
    N=length(lmap.offset);
    imaset=createUnitSet(N);
    for i=1:N
      if lmap.gain(i,i)==Inf
          imaset.interval=replaceDimension(imaset.interval,createFullInterval(1),i);
       else   
          imaset.interval=replaceDimension(imaset.interval,uniIntervalAffine(uniInterval(aset.interval,i),lmap.gain(i,i),lmap.offset(i)),i);
       end
       if isempty(imaset.interval)
          imaset=[];
          break
       end
     end
  else
     imaset=[];
  end   
end 

function imset=imageLinearMap(domset,lmap)
  %image of set domset throug linear map lmap
  %returns a set
  imset=[];
  if !isempty(domset)
    for i=1:length(domset.interval)
      imset=setUnion(imset,imageAtomicLinearMap(atomicSet(domset,i),lmap));
    end
  end  
end



function imset=imagePWLinearMap(domset,mlmap)
  %image of set domset throug PW linear map lmap
  %returns a set
  imset=[];
  t=[];
  if !isempty(domset)
    for i=1:length(mlmap.domain)
      intset=interSet(domset,mlmap.domain(i));
      imset=setUnion(imset,imageLinearMap(intset,mlmap.linearMap(i)));
    end  
  end
end


function pimaset=preImageAtomicLinearMap(aset,lmap)
  %pre Image of atomic set aset through linear map lmap (returns an atomic set)
  N=length(lmap.offset);
  pimaset=[];
  if !isempty(aset)
       maximage=imageAtomicLinearMap(createFullSet(N),lmap); %full image set of lmap
       actualmaximage=interAtomicSet(aset,maximage); %components of aset in the image
       pimaset=imageAtomicLinearMap(actualmaximage,invLinearMap(lmap)); %inverse
  end  
end


function pimset=preImageLinearMap(set1,lmap)
  %pre Image of a set (set1) through linear map lmap (returns a set)
  pimset=[];
  if !isempty(set1)
    for i=1:length(set1.interval)
      pimset=setUnion(pimset,preImageAtomicLinearMap(atomicSet(set1,i),lmap));
    end
  end
end

function pimset=preImagePWLinearMap(set1,pwlmap)
  %pre Image of a set (set1) through piecewise linear map lmap (returns a set)
  pimset=[];
  if !isempty(set1)
    for i=1:length(pwlmap.linearMap)
      pimi=preImageLinearMap(set1,pwlmap.linearMap(i));%pre image through i-th linear map
      pimset=setUnion(pimset,interSet(pimi,pwlmap.domain(i))); %add to preimage the intersection with the i--th corresponding domain
    end
  end
end


function outpwlmap=compMap(pwlmap2,pwlmap1)
 %composition of two pwlinear maps pwlmap1(pwlmap2)
  outpwlmap=[];
  if !isempty(pwlmap1)&&!isempty(pwlmap2)
    for i=1:length(pwlmap1.domain)
      for j=1:length(pwlmap2.domain)
        domaux=imageLinearMap(pwlmap1.domain(i),pwlmap1.linearMap(i)); %maximum image set of pwlmap1
        domaux=interSet(domaux,pwlmap2.domain(j)); %intersection with domain of pwlmap2
        newdom=interSet(pwlmap1.domain(i),preImageLinearMap(domaux,pwlmap1.linearMap(i))); 
        if !isempty(newdom)
          newmap=compLinearMap(pwlmap2.linearMap(j),pwlmap1.linearMap(i));
          if isempty(outpwlmap)
            outpwlmap=createPWLinearMap(newdom,newmap);
          else
            d=length(outpwlmap.domain);
            outpwlmap.domain(d+1)=newdom;
            outpwlmap.linearMap(d+1)=newmap;
          end
        end  
      end
    end
  end  
end

%function pwlmap2=mininvCompactPWLMap(pwlmap1)
%  %computes the mininv map, defined as g(y)=min(x in domain(f)) such that f(x)=y 
%  %pwlmap1 is a compact map (it has a single set domain and a single linear map).
%  N=length(pwlmap1.linearMap.offset); %dimension of the map
%  dominv=imageLinearMap(pwlmap1.domain,pwlmap1.linearMap); %domain of inverse
%  mapinv=invLinearMap(pwlmap1.linearMap);
%  mincomp=minComponentSet(pwlmap1.domain); %minimum component of domain
%  for i=1:N 
%    if mapinv.gain(i,i)==Inf
%      mapinv.gain(i,i)=0;
%      mapinv.offset(i)=mincomp(i); %returns the index of the minimum component
%    end  
%  end
%  pwlmap2=createPWLinearMap(dominv,mapinv);
%end

function pwlmap2=minAtomicPWLMap(dom,lmap1,lmap2)
  %piecewise linear map on atomic set dom with map(x)=min(lmap1(x),lmap2(x))
  pwlmap2=createPWLinearMap(dom,lmap1);%initially, take lmap1
  N=length(lmap1.offset); %map dimension
  for i=1:N
    if lmap1.gain(i,i)!=lmap2.gain(i,i)
       %partition through dimension i
       xpart=(lmap2.offset(i)-lmap1.offset(i))/(lmap1.gain(i,i)-lmap2.gain(i,i)); 
       %both maps are equal at xpart
       if xpart<=dom.interval.start(i)
          %the intersection is before the domain, so the minimum is lmap with minimum gain
          if lmap2.gain(i,i)<lmap1.gain(i,i)
            pwlmap2.linearMap=lmap2;
          end
       elseif xpart>=dom.interval.end(i)
          %the intersection is after the domain, so the minimum is lmap with maximum gain
          if lmap2.gain(i,i)>lmap1.gain(i,i)
            pwlmap2.linearMap=lmap2;
          end
       else
          %the intersection is in the domain, we need to split it.
          inti=uniInterval(dom.interval,i); %interval to split
          int1=createInterval(inti.start,inti.step,floor(xpart)); %first part of the interval
          int2=removeUniInterval(inti,int1); %second part of the interval
          dom1=dom;
          dom2=dom;
          dom1.interval=replaceDimension(dom1.interval,int1,i); %replace i-th dimension with the split interval
          dom2.interval=replaceDimension(dom2.interval,int2,i); %replace i-th dimension with the split interval
          pwlmap2.domain=[dom1, dom2];
          if lmap1.gain(i,i)>lmap2.gain(i,i)
            %lmap1 goes first
            pwlmap2.linearMap=[lmap1, lmap2];
          else
            pwlmap2.linearMap=[lmap2, lmap1];
          end  
       end   
       break;
    elseif lmap1.offset(i)!=lmap2.offset(i)
       %maps differ in dimension i with identical gain
       if lmap1.offset>lmap2.offset
         pwlmap2.linearMap=lmap2; %lmap2 is always the minimum, otherwise lmap1 is always the minimum
       end
       break;   
    end   
  end
end

function pwlmap2=minPWLMap(dom,lmap1,lmap2)
  %piecewise linear map on set dom with map(x)=min(lmap1(x),lmap2(x))
  pwlmap2=minAtomicPWLMap(atomicSet(dom,1),lmap1,lmap2); %start with first atomic set
  for i=2:length(dom.interval)
    mapaux=minAtomicPWLMap(atomicSet(dom,i),lmap1,lmap2); %min on the i-th atomic set
    for k=1:length(mapaux.domain) %mapaux can have two domains 
      if sameMap(mapaux.linearMap(k),pwlmap2.linearMap(1)) %the minimum corresponds to the first domain 
        pwlmap2.domain(1)=setUnion(pwlmap2.domain(1),mapaux.domain(k));
      else %the minimum corresponds to the second domain
        if length(pwlmap2.domain)==1 %we may have only one domain in pwlmap2, so add the second domain
          pwlmap2.domain(2)=mapaux.domain(k);
          pwlmap2.linearMap(2)=mapaux.linearMap(k); 
        else 
          pwlmap2.domain(2)=setUnion(pwlmap2.domain(2),mapaux.domain(k)); 
        end
      end
    end
  end
end

function map3=combinePWLMaps(map1,map2)
  %map3=map1 on map1.domain and map3=map2 on map2.domain\map1.domain
   if isempty(map1)
      map3=map2;
   elseif isempty(map2)
      map3=map1;
   else   
    map3=map1;
    dom1=domPWLMap(map1);
    for i=1:length(map2.domain)
      newdom=setMinus(map2.domain(i),dom1);
      if !isempty(newdom)
        map3.domain(length(map3.domain)+1)=newdom;
        map3.linearMap(length(map3.linearMap)+1)=map2.linearMap(i);
      end
    end
  end  
end


function map3=minMap(map1,map2)
 %minimum between two PWLinear maps
 map3=[];
 if !isempty(map1)&&!isempty(map2)
   for i=1:length(map1.domain)
     for j=1:length(map2.domain)
       dom=interSet(map1.domain(i),map2.domain(j));
       if !isempty(dom)
          mapaux=minPWLMap(dom,map1.linearMap(i),map2.linearMap(j));
          if isempty(map3)
              map3=mapaux;
          else    
            map3=combinePWLMaps(mapaux,map3);
          end  
       end
     end
   end
  end
end


%function pwlmap2=mininvPWLMap(pwlmap1)
%  if !isempty(pwlmap1)
%    mapi=createPWLinearMap(pwlmap1.domain(1),pwlmap1.linearMap(1)); %map of the first domain
%    pwlmap2=mininvCompactPWLMap(mapi); %inverse of first map
%    for i=2:length(pwlmap1.domain) %add inverse of the remaining maps
%      imapi=mininvCompactPWLMap(createPWLinearMap(pwlmap1.domain(i),pwlmap1.linearMap(i))); %inverse of i-th map
%      mapmin=minMap(pwlmap2,imapi); %compute the minimum between imap and pwlmap2
%      pwlmap2=combinePWLMaps(imapi,pwlmap2); %add imapi to pwlmap2
%      if !isempty(mapmin) %if there was intersection between the domain of both maps, then use the minimum in the intersection
%        pwlmap2=combinePWLMaps(mapmin,pwlmap2); 
%      end
%    end  
%  else
%    pwlmap2=[];
%  end  
%end

function dom1=domPWLMap(map1)
 %returns the whole domain of a map
  dom1=map1.domain(1);
 for i=2:length(map1.domain)
   dom1=setUnion(dom1,map1.domain(i));
 end
end


function map2=reduceMapN(map1,d)
%replaces some maps on dimension d so that it converges faster to MapInf on i--th dimension
%gain must be 1 or 0, and offset must be 0 or negative (for contraction)
  map2=map1;
  for i=1:length(map1.domain)
    if map1.linearMap(i).gain(d)==1&&map1.linearMap(i).offset(d)<0 %a map that may imply a large number of iterations
      off=-map1.linearMap(i).offset(d);
      for j=1:length(map1.domain(i).interval) 
         adom=atomicSet(map1.domain(i),j); %check every atomic set
         if (adom.interval.end(d)-adom.interval.start(d))>off^2 
            %it is better to reduce the map to off constant maps 
            % if a map has the form y=x-off, after applying a number of times the map, y will become less than the start of the map
            % so, it is replaced by y = interval.start - off +k with k=0,..,off-1
            % that map is used for all x in [interval.start+k : off : interva.end] 
           % if off is too large, it is not convenient to do this as the resulting number of intervals will become 
           %larger than the iterations needed to reach th beggining of the interval. 
           newmap=[];
           for k=1:off
              newlmap=map1.linearMap(i);
              newlmap.gain(d)=0;
              newlmap.offset(d)=adom.interval.start(d)+k-off-1; 
              newset=adom;
              newinterval=createInterval(adom.interval.start(d)+k-1,off,adom.interval.end(d));
              newset.interval=replaceDimension(newset.interval,newinterval,d);
              if isempty(newmap)
                newmap=createPWLinearMap(newset,newlmap);
              else
                newmap.domain=[newmap.domain,newset];
                newmap.linearMap=[newmap.linearMap,newlmap];                
              end  
           end
           %remove adom and add new domains
           newdomi=removeAtomicSet(map1.domain(i),j);
           if isempty(newdomi)
             %remove domain(i) from map2
             if i<length(map2.domain)
               map2.domain=[map2.domain(1:i-1),map2.domain(i+1:length(map2.domain))];
               map2.linearMap=[map2.linearMap(1:i-1),map2.linearMap(i+1:length(map2.linearMap))];
             else
               map2.domain=[map2.domain(1:i-1)];
               map2.linearMap=[map2.linearMap(1:i-1)];
             end
           else
              map2.domain(i)=newdomi;  
           end
           %add new domains to map2
           for k=1:length(newmap.domain)
             map2.domain=[map2.domain,newmap.domain(k)];
             map2.linearMap=[map2.linearMap,newmap.linearMap(k)];
           end
         end  
      end
    end
  end
end

function map2=mapInf(map1)
  map2=reduceMapN(map1,1);
  for i=2:length(map1.linearMap(1).offset) 
    map2=reduceMapN(map2,i);
  end
  %estimate number of iterations
  maxit=0;
  for i=1:length(map2.domain)
    a= max(diag(map2.linearMap(i).gain) .* abs(map2.linearMap(i).offset)); 
    if a>0  %there is gain and offset in some dimension... we may need to iterate
      its=0;
      for dim=1:length(map2.linearMap(i).offset)
        if map2.linearMap(i).offset(dim)<0&&map2.linearMap(i).gain(dim,dim)==1
          for j=1:length(map2.domain(i).interval) %traverse different intervals
            its=max(its,ceil((map2.domain(i).interval(j).end(dim)-map2.domain(i).interval(j).start(dim))/abs(map2.linearMap(i).offset(dim))));
          end
        end
      end
      %display(its);
      maxit=maxit+its;
    else
      if min(diag(map2.linearMap(i).gain))==0
        maxit=maxit+1; %constant map, it may point to a different domain
      end  
    end      
  end
  maxit=floor(log2(maxit))+1; %we apply map2=map2(map2), so we apply 2^N times map1
  for i=1:maxit
    map2=compMap(map2,map2);
  end
end


%function pwlmap2=mininvFCompactPWLMap(pwlmap1,pwlmap0)
%  %map2(y)=x iff map1(x)=y&& map0(x)<=map0(z) for all z:map1(z)=y
%  %pwlmap1 is a compact map (it has a single set domain and a single linear map).
%  N=length(pwlmap1.linearMap.offset); %dimension of the map
%  dominv=imageLinearMap(pwlmap1.domain,pwlmap1.linearMap); %domain of inverse
%  mapinv=invLinearMap(pwlmap1.linearMap);
%  mindomain=[];
%  for i=1:N 
%    if mapinv.gain(i,i)==Inf %the original map has gain=0 -> we must point to the minimum component
%      if isempty(mindomain) %we compute this only once, when necessary
%        mincomp=minComponentSet(imagePWLinearMap(pwlmap1.domain,pwlmap0)); %minimum component of pwlmap0 in the domain
%        domincomp=preImagePWLinearMap(singleton(mincomp),pwlmap0); %preimage of pwlmap0 for the minimum component
%        mindomain=minComponentSet(domincomp); %minimum element in the preimage (we could have taken any component of the preimage)
%      end
%      mapinv.gain(i,i)=0;
%      mapinv.offset(i)=mindomain(i); %point in the domain for which pwlmap0 is minimum
%    end  
%  end
%  pwlmap2=createPWLinearMap(dominv,mapinv);
%end


function pwlmap2=minAdjCompMap(pwlmap1,pwlmap0)
%giving a compact function map1(x) and a map map0(x), it computes map2(x)=min(map0(z):map1(z)=x)  
  N=length(pwlmap1.linearMap.offset); %dimension of the map
  dominv=imageLinearMap(pwlmap1.domain,pwlmap1.linearMap); %domain of inverse
  mapinv=invLinearMap(pwlmap1.linearMap);
  pwlmapinv=createPWLinearMap(dominv,mapinv); %inverse map of map1 
  if max(diag(mapinv.gain))<Inf %bijective map
       pwlmap2=compMap(pwlmap0,pwlmapinv);
  elseif min(diag(mapinv.gain))==Inf %constant map
    mincomp=minComponentSet(imagePWLinearMap(pwlmap1.domain,pwlmap0)); %minimum component of pwlmap0 in the domain
    pwlmap2=pwlmapinv;
    pwlmap2.linearMap.gain=zeros(N,N); %gain 0
    pwlmap2.linearMap.offset=mincomp'; %the map goes to mincomp 
  else  %bijective is some dimension and constant in other
    mincomp=minComponentSet(imagePWLinearMap(pwlmap1.domain,pwlmap0)); %minimum component of pwlmap0 in the domain
    mindom=minComponentSet(pwlmap1.domain);
    for i=1:N %replace inverse of constant maps
      if mapinv.gain(i,i)==Inf %the original map has gain=0 -> we must point to the minimum component
        pwlmapinv.linearMap(1).gain(i,i)=0;
        pwlmapinv.linearMap(1).offset(i)=mindom(i); %for later replacement
      end  
    end
    pwlmap2=compMap(pwlmap0,pwlmapinv);
    for i=1:N %replace inverse of constant maps
      if mapinv.gain(i,i)==Inf %the original map has gain=0 -> we must point to the minimum component
        pwlmap2.linearMap(1).gain(i,i)=0;
        pwlmap2.linearMap(1).offset(i)=mincomp(i); %minimum value of pwlmap0
      end  
    end
  end  
end

function pwlmap2=minAdjMap(pwlmap1,pwlmap0)
%giving two functions map1(x),map0(x), it computes map2(x)=min(map0(z):map1(z)=x)
  if !isempty(pwlmap1)
    mapi=createPWLinearMap(pwlmap1.domain(1),pwlmap1.linearMap(1)); %map of the first domain
    pwlmap2=minAdjCompMap(mapi,pwlmap0); %first map
    for i=2:length(pwlmap1.domain) %add remaining maps
      mapi=createPWLinearMap(pwlmap1.domain(i),pwlmap1.linearMap(i)); %map of the first domain
      map2=minAdjCompMap(mapi,pwlmap0); %i-th adj map
      minmap=minMap(pwlmap2,map2); %minimum between current pwlmap2 and the new adj map.
      pwlmap2=combinePWLMaps(map2,pwlmap2); 
      if !isempty(minmap) %there is intersection between maps
         pwlmap2=combinePWLMaps(minmap,pwlmap2); %we use minmap in its domain
      end      
    end
  else
    pwlmap2=[];
  end  
end

%function pwlmap2=minAdjMap(pwlmap1,pwlmap0)
%%giving two functions map1(x),map0(x), it computes map2(x)=min(map0(z):map1(z)=x)
%  if !isempty(pwlmap1)
%    mapi=createPWLinearMap(pwlmap1.domain(1),pwlmap1.linearMap(1)); %map of the first domain
%    imapi=mininvFCompactPWLMap(mapi,pwlmap0); %inverse of first map minimizing pwlmap0
%    pwlmap2=compMap(pwlmap0,imapi); %first map
%    for i=2:length(pwlmap1.domain) %add remaining maps
%      imapi=mininvFCompactPWLMap(createPWLinearMap(pwlmap1.domain(i),pwlmap1.linearMap(i)),pwlmap0); %inverse of i-th map  
%      map2=compMap(pwlmap0,imapi); %i-th adj map
%      minmap=minMap(pwlmap2,map2); %minimum between current pwlmap2 and the new adj map.
%      pwlmap2=combinePWLMaps(map2,pwlmap2); 
%      if !isempty(minmap) %there is intersection between maps
%         pwlmap2=combinePWLMaps(minmap,pwlmap2); %we use minmap in its domain
%      end      
%    end
%  else
%    pwlmap2=[];
%  end  
%end

function map2=atomizeMap(map1)
  %domain partition into atomic sets
  map2=[];
  for i=1:length(map1.domain) %traverse set
    for j=1:length(map1.domain(i).interval) %traverse atomic sets
      if !isempty(map2)
        map2.domain=[map2.domain,atomicSet(map1.domain(i),j)];
        map2.linearMap=[map2.linearMap,map1.linearMap(i)];
      else
        map2=createPWLinearMap(atomicSet(map1.domain(i),j),map1.linearMap(i));
      end
    end
  end
end

%function map3=compMap(map1,map2)
%  map3=compPWLinearMap(map2,map1);
%end

